import React, { useCallback, useEffect, useMemo, useState } from 'react';
import { Provider, Preview, Error } from 'jarle';
import MonacoEditor from '@monaco-editor/react';
import { styled, css } from '@storybook/theming';
import { FaAngleRight } from '@react-icons/all-files/fa/FaAngleRight';
import { FaCopy } from '@react-icons/all-files/fa/FaCopy';
import { FaCheck } from '@react-icons/all-files/fa/FaCheck';
import copy from 'copy-to-clipboard';
import { AutoTypings } from '@hirohe/monaco-editor-auto-typings';
import { v4 as uuid } from 'uuid';
import EditorWrapper from './EditorWrapper';
import applyJsxHighlighter from '../utils/apply-jsx-highlighter';
import { defaultEditorOptions } from '../constants';
const Wrapper = styled.div({
    width: '100%',
    overflow: 'hidden',
    borderRadius: '8px',
    border: '1px solid #eaeaea',
});
const LiveContent = styled.div({
    height: '100%',
    padding: '16px',
    overflow: 'auto',
});
const summaryBarCss = css `
  color: #666;
  font-size: 14px;
  height: 42px;
  background: #f5f5f5;
  border-bottom: 1px solid #eaeaea;
  display: flex;
  align-items: center;
  justify-content: space-between;
  padding: 0 16px;
  > div {
    display: flex;
    align-items: center;
  }
  .icon-btn {
    box-sizing: content-box;
    padding: 4px;
    cursor: pointer;
    margin-right: 8px;
    transition: all 0.2s ease;
    &:hover {
      color: #333;
    }
  }
  &.open {
    .icon-btn.open-btn {
      transform: rotate(90deg);
    }
  }

  .copy-result-text {
    color: limegreen;
    margin-right: 8px;
    font-size: 12px;
  }
`;
const SummaryBar = styled.div `
  ${summaryBarCss}
`;
const editorContentCss = css `
  max-height: 1000px;
  overflow: hidden;
`;
const EditorContent = styled.div `
  ${editorContentCss}
`;
const Playground = ({ code, autoTypings = false, defaultExpanded = false, scope, providerProps, resolveTypeDefinition, editorProps, className, }) => {
    const editorRef = React.useRef();
    const monacoRef = React.useRef();
    const [_code, setCode] = useState(code);
    const [open, setOpen] = useState(defaultExpanded);
    const [copied, setCopied] = useState(false);
    const [editorContentHeight, setEditorContentHeight] = useState(0);
    // https://github.com/suren-atoyan/monaco-react/issues/354
    const filenameRef = React.useRef(`main-${uuid()}.tsx`);
    const onCopy = useCallback((e) => {
        e.stopPropagation();
        e.preventDefault();
        copy(_code);
        setCopied(true);
    }, []);
    const onCodeChange = useCallback((newCode) => {
        setCode(newCode);
    }, []);
    const monacoConfiguration = useCallback(() => {
        const _monaco = monacoRef.current;
        if (_monaco) {
            // jsx and tsx syntax config
            _monaco.languages.typescript.typescriptDefaults.setCompilerOptions(Object.assign(Object.assign({}, _monaco.languages.typescript.typescriptDefaults.getCompilerOptions()), { target: _monaco.languages.typescript.ScriptTarget.Latest, jsx: _monaco.languages.typescript.JsxEmit.Preserve }));
            if (autoTypings) {
                _monaco.languages.typescript.typescriptDefaults.setDiagnosticsOptions({
                    noSemanticValidation: false,
                    noSyntaxValidation: false,
                });
            }
            else {
                _monaco.languages.typescript.typescriptDefaults.setDiagnosticsOptions({
                    noSemanticValidation: true,
                    noSyntaxValidation: true,
                });
            }
        }
    }, [autoTypings]);
    const beforeMount = useCallback((_monaco) => {
        monacoRef.current = _monaco;
        monacoConfiguration();
    }, []);
    const onMount = useCallback((editor, _monaco) => {
        editorRef.current = editor;
        editor.onDidContentSizeChange((e) => {
            setEditorContentHeight(e.contentHeight);
            editor.layout();
        });
        if (autoTypings) {
            const autoTypings = AutoTypings.create(editor, {
                monaco: _monaco,
                resolveTypeDefinition,
            });
            // preload type definitions of React, and global variables definitions provided by Jarle
            autoTypings
                .resolvePackage({
                kind: 'package',
                packageName: 'react',
                importPath: '',
            })
                .then(() => {
                _monaco.languages.typescript.typescriptDefaults.addExtraLib(`
            declare global {
              const React: typeof React;
              const useState: typeof React.useState;
              const useEffect: typeof React.useEffect;
              const useRef: typeof React.useRef;
              const useCallback: typeof React.useCallback;
              const useMemo: typeof React.useMemo;
              const useContext: typeof React.useContext;
              const useReducer: typeof React.useReducer;
              const useImperativeHandle: typeof React.useImperativeHandle;
              const useLayoutEffect: typeof React.useLayoutEffect;
              const useDebugValue: typeof React.useDebugValue;
              
              const render: (element: React.ReactNode) => void;
            }
            
            export {};
          `, 'inmemory://model/global.d.ts');
            });
        }
        applyJsxHighlighter(_monaco, editor);
    }, [autoTypings, resolveTypeDefinition]);
    useEffect(() => {
        var _a;
        (_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.layout();
    }, [editorContentHeight]);
    useEffect(() => {
        if (copied) {
            setTimeout(() => {
                setCopied(false);
            }, 1000);
        }
    }, [copied]);
    useEffect(() => {
        if (open) {
            if (editorContentHeight === 0) {
                // TODO maybe had a better way to update editor content height
                setTimeout(() => {
                    var _a;
                    setEditorContentHeight((_a = editorRef.current) === null || _a === void 0 ? void 0 : _a.getContentHeight());
                }, 200);
            }
        }
        else {
            setEditorContentHeight(0);
        }
    }, [open, editorContentHeight]);
    const defaultLanguage = useMemo(() => {
        return autoTypings ? 'typescript' : 'javascript';
    }, [autoTypings]);
    return (React.createElement(Wrapper, { className: className },
        React.createElement(LiveContent, null,
            React.createElement(Provider, Object.assign({ language: "typescript", scope: scope }, providerProps, { code: _code }),
                React.createElement(Preview, null),
                React.createElement(Error, null))),
        React.createElement(SummaryBar, { className: open ? 'open' : '' },
            React.createElement("div", null,
                React.createElement(FaAngleRight, { className: "icon-btn open-btn", onClick: () => setOpen((state) => !state) }),
                React.createElement("span", null, "Code Editor")),
            React.createElement("div", null,
                copied && (React.createElement("span", { className: "copy-result-text" },
                    "Copied !",
                    React.createElement(FaCheck, null))),
                React.createElement(FaCopy, { className: "icon-btn", title: "Copy the code", onClick: onCopy }))),
        React.createElement(EditorContent, { className: open ? 'open' : '', style: { height: editorContentHeight } },
            React.createElement(EditorWrapper, null,
                React.createElement(MonacoEditor, Object.assign({ width: "100%", theme: "vs-dark", language: defaultLanguage, options: defaultEditorOptions }, editorProps, { path: `inmemory://model/${filenameRef.current}`, value: _code, onChange: onCodeChange, beforeMount: beforeMount, onMount: onMount }))))));
};
export default Playground;
